from typing import Optional, Union

from globus_sdk import ClientCredentialsAuthorizer, RefreshTokenAuthorizer

from gcs_cli import version
from gcs_cli.config import internal_auth_client, token_storage_adapter
from gcs_cli.sdk_client import AuthClient
import gcs_cli.helpers.client_login as client_login


def _build_token_authorizer() -> Optional[RefreshTokenAuthorizer]:
    adapter = token_storage_adapter()
    token_data = adapter.get_token_data("auth.globus.org")
    authorizer = None

    # if there's token data, use it to build the authorizer
    if token_data is not None:
        authorizer = RefreshTokenAuthorizer(
            token_data["refresh_token"],
            internal_auth_client(),
            access_token=token_data["access_token"],
            expires_at=token_data["expires_at_seconds"],
            on_refresh=adapter.on_refresh,
        )

    return authorizer


def get_client() -> AuthClient:
    authorizer: Union[Optional[RefreshTokenAuthorizer], ClientCredentialsAuthorizer]
    if client_login.is_client_login():
        authorizer = client_login.get_auth_authorizer()
    else:
        authorizer = _build_token_authorizer()

    return AuthClient(authorizer=authorizer, app_name=version.app_name)
