from typing import overload, Optional, Type, Union
from typing_extensions import Literal

import globus_sdk

from ..config import internal_auth_client, token_storage_adapter


@overload
def get_api(service: Literal["auth.globus.org"]) -> globus_sdk.AuthClient:
    ...


@overload
def get_api(
    service: Literal["auth.globus.org"], *, environment: Optional[str]
) -> globus_sdk.AuthClient:
    ...


@overload
def get_api(
    service: Literal["auth.globus.org"],
    *,
    cls: Type[globus_sdk.AuthClient],
) -> globus_sdk.AuthClient:
    ...


@overload
def get_api(
    service: Literal["auth.globus.org"],
    *,
    environment: Optional[str],
    cls: Type[globus_sdk.AuthClient],
) -> globus_sdk.AuthClient:
    ...


@overload
def get_api(service: Literal["transfer.api.globus.org"]) -> globus_sdk.TransferClient:
    ...


@overload
def get_api(
    service: Literal["transfer.api.globus.org"], *, environment: Optional[str]
) -> globus_sdk.TransferClient:
    ...


@overload
def get_api(
    service: Literal["transfer.api.globus.org"],
    *,
    cls: Type[globus_sdk.TransferClient],
) -> globus_sdk.TransferClient:
    ...


@overload
def get_api(
    service: Literal["transfer.api.globus.org"],
    *,
    environment: Optional[str],
    cls: Type[globus_sdk.TransferClient],
) -> globus_sdk.TransferClient:
    ...


@overload
def get_api() -> globus_sdk.TransferClient:
    ...


def get_api(
    service: Optional[str] = None,
    *,
    environment: Optional[str] = None,
    cls: Optional[Type[globus_sdk.BaseClient]] = None,
) -> Union[globus_sdk.AuthClient, globus_sdk.TransferClient]:
    adapter = token_storage_adapter()

    token_data = adapter.get_token_data(service or "transfer.api.globus.org")
    authorizer = None

    if service is None:
        service = "transfer.api.globus.org"

    # if there's token data, use it to build the authorizer
    if token_data is not None:
        authorizer = globus_sdk.RefreshTokenAuthorizer(
            token_data["refresh_token"],
            internal_auth_client(),
            access_token=token_data["access_token"],
            expires_at=token_data["expires_at_seconds"],
            on_refresh=adapter.on_refresh,
        )

    if service == "transfer.api.globus.org":
        if not cls:
            cls = globus_sdk.TransferClient
        assert cls is not None
        assert issubclass(cls, globus_sdk.TransferClient)
        return cls(authorizer=authorizer, environment=environment)
    elif service == "auth.globus.org":
        if not cls:
            cls = globus_sdk.AuthClient
        assert cls is not None
        assert issubclass(cls, globus_sdk.AuthClient)
        return cls(authorizer=authorizer, environment=environment)
    raise Exception("Unsupported service")
