from typing import List

from globus.manager.constants import ConfigurationObjectTag
from .oidc_user_credential import OIDCUserCredential


class OneDriveUserCredential(OIDCUserCredential):
    POLICY_DATA_TYPE = "onedrive_user_credential_policies#1.0.0"
    POLICY_DATA_TYPES = [POLICY_DATA_TYPE]
    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.USER_CREDENTIAL.name
            + "28ef55da-1f97-11eb-bdfd-12704e0d6a4d"
        )
    }

    @property
    def connector_properties(self) -> List[str]:
        return super().connector_properties + ["tid"]

    def _credentials_to_patch(self, credentials):
        patch = super()._credentials_to_patch(credentials)
        if credentials:
            id_token = credentials.id_token
            if "policies" not in patch:
                patch["policies"] = {"DATA_TYPE": self.POLICY_DATA_TYPE}
            if "tid" in id_token:
                patch["policies"]["tid"] = id_token["tid"]
        return patch

    def __init__(
        self, appsync_data=None, doc=None, storage_gateway=None, credentials=None
    ):
        if doc:
            self.username = doc["username"]
            new_doc = {**doc}
            new_doc.pop("display_name", None)
            # Policies must be either empty (for an unprovisioned credential)
            # or passed in as a credentials object from the microsoft callback
            policies = new_doc.get("policies", None)
            if policies is not None and policies != {
                "DATA_TYPE": self.POLICY_DATA_TYPE
            }:
                raise ValueError("policies")
            doc = new_doc

        super().__init__(
            appsync_data=appsync_data, doc=doc, storage_gateway=storage_gateway
        )
        if doc:
            patch = self._credentials_to_patch(credentials)
            patch["deleted"] = False
            patch["provisioned"] = bool(credentials)
            self._patch(**patch)

    tid = OIDCUserCredential.connector_property("tid")

    @property
    def connector_id(self):
        return "28ef55da-1f97-11eb-bdfd-12704e0d6a4d"

    @connector_id.setter
    def connector_id(self, value):
        pass

    @property
    def token_uri(self) -> str:
        return "https://login.microsoftonline.com/{}/oauth2/v2.0/token".format(
            getattr(self.storage_gateway, "tenant", "common")
        )

    @property
    def supports_revoke(self) -> bool:
        return False

    def __repr__(self):
        return "<OneDriveUserCredential {}>".format(self.id)
