from globus.manager.api.errors import InternalServerError


class Result:
    """
    Class representing the Response data type.
    """

    DATA_TYPE = "result#1.0.0"

    def __init__(
        self,
        http_response_code=200,
        code=None,
        detail=None,
        message=None,
        data=None,
        marker=None,
    ):
        """
        :param http_response_code:
            The HTTP response code. If it is not specified, the default of
            200 is supplied.
        :param code:
            The string code for this type of error per the API
            specification.
        :param detail:
            A string or object containing machine-readable information to
            interpret this result.
        :param message:
            A human-readable string containing information about this
            result.
        :param data:
            A list of data items to serialize in this response.
        :param marker:
            A pagination marker if this API response contains a partial
            data response.
        """
        self.http_response_code = http_response_code

        if code is not None:
            self.code = code
        elif http_response_code == 200:
            self.code = "success"
        elif http_response_code == 201:
            self.code = "created"
        elif http_response_code == 202:
            self.code = "accepted"
        else:
            raise InternalServerError()

        if detail is not None:
            self.detail = detail
        else:
            self.detail = self.code

        if message:
            self.message = message

        if data is not None:
            self.data = data
        if marker:
            self.marker = marker.encode()
        self.has_next_page = marker is not None
