import marshmallow as ma
from .data_type import DataTypeField, DataTypeSchema
from .semantic_version import semantic_version_expression


class Connector(ma.Schema):
    """
    Connector information document
    """

    _DATA_TYPE = "connector#1.0.0"
    DATA_TYPE = DataTypeField(_DATA_TYPE)
    display_name = ma.fields.String(
        required=True,
        description="Friendly name of the connector",
        example="POSIX",
    )
    id = ma.fields.String(required=True, description="Unique id of this connector type")
    version = ma.fields.String(
        validate=[ma.validate.Regexp(semantic_version_expression)],
        description="Semantic version of this connector implementation",
    )


connector_schema = Connector()

DataTypeSchema.register_schema(connector_schema)
