import uuid

import marshmallow as ma

from .data_type import DataTypeField, DataTypeSchema
from .schema_version_checker import verify_schema_version


class EndpointSubscription(ma.Schema):
    _DATA_TYPE = "endpoint_subscription#1.0.0"

    DATA_TYPE = DataTypeField(_DATA_TYPE)
    subscription_id = ma.fields.String(
        required=True,
        allow_none=True,
        description="""
        Either the id of a Globus subscription or the special value
        "DEFAULT" if the caller has only one subscription associated with
        their identity set.
        """,
    )

    @ma.validates("DATA_TYPE")
    def validate_data_type(self, data, **kwargs):
        verify_schema_version(data, self._DATA_TYPE)

    @ma.validates("subscription_id")
    def validate_subscription_id(self, data, **kwargs):
        if data:
            if data == "DEFAULT":
                return data
            else:
                try:
                    return uuid.UUID(data)
                except Exception:
                    raise ma.ValidationError('Value must be a UUID or "DEFAULT"')


endpoint_subscription_schema = EndpointSubscription()
DataTypeSchema.register_schema(endpoint_subscription_schema)
