import marshmallow as ma

from .semantic_version import semantic_version_expression
from .data_type import DataTypeField, DataTypeSchema


class Info(ma.Schema):
    """
    This document contains information about the Globus Connect
    Server, including its software and supported API version
    number.
    """

    _DATA_TYPE = "info#1.0.0"
    DATA_TYPE = DataTypeField(_DATA_TYPE)
    manager_version = ma.fields.Str(
        validate=[ma.validate.Regexp(semantic_version_expression)],
        description="Globus Connect Server software version",
    )
    api_version = ma.fields.Str(
        validate=[ma.validate.Regexp(semantic_version_expression)],
        description="Semantic version of the Globus Connect Server API",
    )
    client_id = ma.fields.UUID(description="Client id that created the endpoint")
    endpoint_id = ma.fields.UUID(
        description="Transfer endpoint ID managed by this GCS Manager"
    )
    domain_name = ma.fields.String(
        description="Domain name for the GCS Manager service"
    )


info_schema = Info()

DataTypeSchema.register_schema(info_schema)
