"""Add last_access to collection

Revision ID: 821944572cf4
Revises: ab11098d41d5
Create Date: 2023-03-27 09:57:12.291795

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "821944572cf4"
down_revision = "ab11098d41d5"
branch_labels = None
depends_on = None


def upgrade():
    with op.batch_alter_table("collection", schema=None) as batch_op:
        batch_op.add_column(sa.Column("last_access", sa.Date(), nullable=True))
    with op.batch_alter_table("pending_updates", schema=None) as batch_op:
        batch_op.alter_column(
            "update_type",
            type_=sa.Enum(
                "DELETE",
                "UPDATE_DOMAIN",
                "PATCH",
                "UPDATE_LAST_ACCESS",
                name="pendingupdatetype",
            ),
            nullable=False,
        )


def downgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    with op.batch_alter_table("collection", schema=None) as batch_op:
        batch_op.drop_column("last_access")
    op.execute(
        "DELETE FROM pending_updates WHERE update_type = 'UPDATE_LAST_ACCESS'"
    )
    with op.batch_alter_table("pending_updates", schema=None) as batch_op:
        batch_op.alter_column(
            "update_type",
            type_=sa.Enum("DELETE", "UPDATE_DOMAIN", "PATCH", name="pendingupdatetype"),
            nullable=False,
        )
