"""Add endpoint:owner role

Revision ID: f375c2347d13
Revises: fd24f05ab665
Create Date: 2019-11-22 08:07:46.537624

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'f375c2347d13'
down_revision = 'fd24f05ab665'
branch_labels = None
depends_on = None


def upgrade():
    # Backup role table
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    op.rename_table("role", "role_old")
    # recreate with new enum value
    op.create_table("role",
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('principal_type', sa.Enum('identity', 'group', name='principaltype'), nullable=False),
        sa.Column('identity_id', sa.TEXT(), nullable=False),
        sa.Column('role', sa.Enum('endpoint_owner', 'endpoint_administrator', 'endpoint_activity_manager', 'endpoint_activity_monitor', 'collection_administrator', 'collection_activity_manager', 'collection_activity_monitor', 'collection_access_manager', name='roletype'), nullable=False),
        sa.Column('resource', sa.TEXT(), nullable=True),
        sa.Column('primary_role', sa.BOOLEAN(), server_default='0', nullable=False),
        sa.PrimaryKeyConstraint('id'),
        sa.ForeignKeyConstraint(['resource'], ['collection.id']),
        sa.UniqueConstraint('principal_type', 'identity_id', 'role', 'resource')
        )
    # Copy data into new role table
    op.execute(
        """
        INSERT into role
        (id, principal_type, identity_id, role, resource)
        SELECT id, principal_type, identity_id, role, resource from role_old
        """)
    # drop old table
    op.drop_table('role_old')


def downgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    # Backup role table
    op.rename_table("role", "role_old")
    # recreate with new enum value
    op.create_table("role",
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('principal_type', sa.Enum('identity', 'group', name='principaltype'), nullable=False),
        sa.Column('identity_id', sa.TEXT(), nullable=False),
        sa.Column('role', sa.Enum('endpoint_administrator', 'endpoint_activity_manager', 'endpoint_activity_monitor', 'collection_administrator', 'collection_activity_manager', 'collection_activity_monitor', 'collection_access_manager', name='roletype'), nullable=False),
        sa.Column('resource', sa.TEXT(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
        sa.ForeignKeyConstraint(['resource'], ['collection.id']),
        sa.UniqueConstraint('principal_type', 'identity_id', 'role', 'resource')
        )
    # Copy data into new role table
    op.execute(
        """
        INSERT into role
        (id, principal_type, identity_id, role, resource)
        SELECT id, principal_type, identity_id, role, resource from role_old
        WHERE role != 'endpoint_owner'
        """)
    # drop old table
    op.drop_table('role_old')
